/*
 * Copyright (c) 2002-2009, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.directory.business;

import fr.paris.lutece.plugins.directory.utils.DirectoryUtils;
import fr.paris.lutece.portal.business.workflow.State;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.workgroup.AdminWorkgroupResource;
import fr.paris.lutece.util.date.DateUtil;
import fr.paris.lutece.util.xml.XmlUtil;

import java.sql.Timestamp;

import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;


/**
 *
 * class Record
 *
 */
public class Record implements AdminWorkgroupResource
{
    public static final String WORKFLOW_RESOURCE_TYPE = "DIRECTORY_RECORD";
    private static final String TAG_RECORD = "record";
    private static final String ATTRIBUTE_RECORD_ID = "id";
    public static final String TAG_LIST_RECORD_FIELD = "list-record-field";
    public static final String TAG_RECORD_FIELD = "record-field";
    public static final String TAG_RECORD_FIELD_VALUE = "record-field-value";
    public static final String TAG_CREATION_DATE = "creation-date";
    public static final String TAG_STATUS = "status";
    private static final String TAG_X = "x";
    private static final String TAG_Y = "y";
    private int _nIdRecord;
    private Timestamp _tDateCreation;
    private Directory _directory;
    private List<RecordField> _listRecordField;
    private List<DirectoryAction> _listAction;
    private boolean _bIsEnabled;
    private String _strRoleKey;
    private String _strWorkgroupKey;
    private Double _fltX = 0.0;
    private Double _fltY = 0.0;

    /**
     *
     * @return the directory associate to the record
     */
    public Directory getDirectory(  )
    {
        return _directory;
    }

    /**
     *
     * set  the directory associate to the record
     * @param directory the directory associate to the record
     */
    public void setDirectory( Directory directory )
    {
        this._directory = directory;
    }

    /**
     * return the id of the record
     * @return the id of the record
     */
    public int getIdRecord(  )
    {
        return _nIdRecord;
    }

    /**
     * set the id of the record
     * @param idRecord the id of the record
     */
    public void setIdRecord( int idRecord )
    {
        _nIdRecord = idRecord;
    }

    /**
     *  return the date creation
     * @return the date creation
     */
    public Timestamp getDateCreation(  )
    {
        return _tDateCreation;
    }

    /**
     * set the date creation
     * @param dateCreation date creation
     */
    public void setDateCreation( Timestamp dateCreation )
    {
        _tDateCreation = dateCreation;
    }

    /**
     *
     * @return the list of field  associate to the record
     */
    public List<RecordField> getListRecordField(  )
    {
        return _listRecordField;
    }

    /**
     * set the list of record field associate to the record
     * @param listRecordField the list of response associate to the form submit
     */
    public void setActions( List<DirectoryAction> listAction )
    {
        _listAction = listAction;
    }

    /**
    *
    * @return the list of field  associate to the record
    */
    public List<DirectoryAction> getActions(  )
    {
        return _listAction;
    }

    /**
     * set the list of record field associate to the record
     * @param listRecordField the list of response associate to the form submit
     */
    public void setListRecordField( List<RecordField> listRecordField )
    {
        _listRecordField = listRecordField;
    }

    /**
     *
     * @return true if the record is enabled
     */
    public boolean isEnabled(  )
    {
        return _bIsEnabled;
    }

    /**
     * set  true if the record is enabled
     * @param enable  true if the record is enabled
     */
    public void setEnabled( boolean enable )
    {
        _bIsEnabled = enable;
    }

    /**
     * Gets the record  role
     * @return recors's role as a String
     *
     */
    public String getRoleKey(  )
    {
        return _strRoleKey;
    }

    /**
     * Sets the directory's role
     * @param strRole The role
     *
     */
    public void setRoleKey( String strRole )
    {
        _strRoleKey = strRole;
    }

    /**
    *
    * @return the work group associate to the record
    */
    public String getWorkgroup(  )
    {
        return _strWorkgroupKey;
    }

    /**
     * set  the work group associate to the record
     * @param workGroup  the work group associate to the record
     */
    public void setWorkgroup( String workGroup )
    {
        _strWorkgroupKey = workGroup;
    }

    /**
     * The Xml of the record
     * @param record the record
     * @param plugin the plugin
     * @param locale the locale
     * @param bWithHtmlCode true if the xml must contain html code like <img src ="">
     * @param state the state of the record
     * @param listEntryResultSearch the list of entry to display
     * @param true if template front or false if template back
     * @return xml
     */
    public StringBuffer getXml( Plugin plugin, Locale locale, boolean bWithHtmlCode, State state,
        List<IEntry> listEntryResultSearch, boolean bDisplayTitleEntryTypeSelect, boolean bDisplayFront,
        boolean bDisplayExport )
    {
        StringBuffer strXml = new StringBuffer(  );
        HashMap<String, String> model = new HashMap<String, String>(  );
        model.put( ATTRIBUTE_RECORD_ID, String.valueOf( this.getIdRecord(  ) ) );
        model.put( TAG_X, String.valueOf( this.getX(  ) ) );
        model.put( TAG_Y, String.valueOf( this.getY(  ) ) );
        XmlUtil.beginElement( strXml, TAG_RECORD, model );

        XmlUtil.addElement( strXml, TAG_CREATION_DATE, DateUtil.getDateString( this.getDateCreation(  ), locale ) );

        if ( state != null )
        {
            XmlUtil.addElement( strXml, TAG_STATUS, state.getName(  ) );
        }

        XmlUtil.beginElement( strXml, Entry.TAG_LIST_ENTRY );

        Map<String, List<RecordField>> mapEntryRecordFields = DirectoryUtils.getSpecificMapIdEntryListRecordField( listEntryResultSearch,
                this.getIdRecord(  ), plugin );

        for ( IEntry entry : listEntryResultSearch )
        {
            if ( entry.getEntryType(  ).getGroup(  ) && ( entry.getChildren(  ) != null ) )
            {
                for ( IEntry entryChildren : entry.getChildren(  ) )
                {
                    if ( !entryChildren.getEntryType(  ).getComment(  ) )
                    {
                        getXmlListRecordField( entryChildren, strXml, plugin, locale, bWithHtmlCode,
                            mapEntryRecordFields.get( Integer.toString( entryChildren.getIdEntry(  ) ) ),
                            bDisplayTitleEntryTypeSelect, bDisplayFront, bDisplayExport );
                    }
                }
            }
            else if ( !entry.getEntryType(  ).getComment(  ) )
            {
                getXmlListRecordField( entry, strXml, plugin, locale, bWithHtmlCode,
                    mapEntryRecordFields.get( Integer.toString( entry.getIdEntry(  ) ) ), bDisplayTitleEntryTypeSelect,
                    bDisplayFront, bDisplayExport );
            }
        }

        XmlUtil.endElement( strXml, Entry.TAG_LIST_ENTRY );
        XmlUtil.endElement( strXml, TAG_RECORD );

        return strXml;
    }

    /**
     * The Xml of the record field list
     * @param entry the entry
     * @param strXml the xml buffer
     * @param plugin the plugin
     * @param locale the locale
     * @param bWithHtmlCode true if the xml must contain html code like <img src ="">
     * @param listRecordField the list of recordField to display
     * @param true if template front or false if template back
     */
    private void getXmlListRecordField( IEntry entry, StringBuffer strXml, Plugin plugin, Locale locale,
        boolean bWithHtmlCode, List<RecordField> listRecordField, boolean bDisplayTitleEntryTypeSelect,
        boolean bDisplayFront, boolean bDisplayExport )
    {
        HashMap<String, String> model = new HashMap<String, String>(  );
        model.put( Entry.ATTRIBUTE_ENTRY_ID, String.valueOf( entry.getIdEntry(  ) ) );
        XmlUtil.beginElement( strXml, Entry.TAG_ENTRY, model );
        XmlUtil.beginElement( strXml, TAG_LIST_RECORD_FIELD );

        if ( listRecordField != null )
        {
            for ( RecordField recordField : listRecordField )
            {
                XmlUtil.beginElement( strXml, TAG_RECORD_FIELD );

                if ( bWithHtmlCode )
                {
                    XmlUtil.addElementHtml( strXml, TAG_RECORD_FIELD_VALUE,
                        recordField.getEntry(  ).getHtmlRecordFieldValue( locale, recordField, bDisplayFront ) );
                }
                else
                {
                    if ( recordField.getFile(  ) != null )
                    {
                        IEntry entryFile = EntryHome.findByPrimaryKey( recordField.getEntry(  ).getIdEntry(  ), plugin );
                        XmlUtil.addEmptyElement( strXml, TAG_RECORD_FIELD_VALUE, null );
                        strXml.append( recordField.getFile(  )
                                                  .getXml( plugin, locale, entry.getEntryType(  ).getIdType(  ),
                                entryFile.getDisplayWidth(  ), entryFile.getDisplayHeight(  ) ) );
                    }
                    else
                    {
                        if ( bDisplayTitleEntryTypeSelect )
                        {
                            XmlUtil.addElementHtml( strXml, TAG_RECORD_FIELD_VALUE,
                                DirectoryUtils.substituteSpecialCaractersForExport( 
                                    recordField.getEntry(  )
                                               .convertRecordFieldTitleToString( recordField, locale, bDisplayFront ) ) );
                        }
                        else
                        {
                            XmlUtil.addElementHtml( strXml, TAG_RECORD_FIELD_VALUE,
                                DirectoryUtils.substituteSpecialCaractersForExport( 
                                    recordField.getEntry(  )
                                               .convertRecordFieldValueToString( recordField, locale, bDisplayFront,
                                        bDisplayExport ) ) );
                        }

                        XmlUtil.addEmptyElement( strXml, File.TAG_FILE, null );
                    }
                }

                XmlUtil.endElement( strXml, TAG_RECORD_FIELD );
            }
        }

        XmlUtil.endElement( strXml, TAG_LIST_RECORD_FIELD );
        XmlUtil.endElement( strXml, Entry.TAG_ENTRY );
    }

    /**
     * Returns the X
     * @return the _fltX
     */
    public Double getX(  )
    {
        return _fltX;
    }

    /**
     * Sets the X
     * @param fltX the _fltX to set
     */
    public void setX( Double fltX )
    {
        _fltX = fltX;
    }

    /**
     * Returns the Y (lattitude)
     * @return the _fltY
     */
    public Double getY(  )
    {
        return _fltY;
    }

    /**
     * Sets the Y (lattitude)
     * @param fltY the _fltY to set
     */
    public void setY( Double fltY )
    {
        _fltY = fltY;
    }
}
